//
//  AABBtree.h
//  ParchmentFlattener
//
//  Copyright (c) 2014 Kazim Pal.
//  Released under the MIT License.
//
#pragma once
#include <Eigen/Dense>
#include <qpoint.h>
#include <qvector.h>
#include <qvector3d.h>

class IGLMesh;

using namespace Eigen;

struct AABBTreeInfo
{
    //heuristics
    int max_tree_depth;
    int curr_max_depth;

    //min number of vertices each leaf can store
    int min_faces;
};

class AABBNode
{
public:
    AABBNode(void);
    AABBNode(QVector<int> fs, AABBTreeInfo &treeInfo, int depth, IGLMesh *mesh);
    ~AABBNode(void);

    Vector3d p_max, p_min;
    AABBNode *left, *right;
    QVector<int> node_faces;
    int numFaces;
    bool isLeaf;

    int FindBestAxis(QVector<int> faceList, IGLMesh *mesh);
    void BuildTree(QVector<int> faceList, AABBTreeInfo &treeInfo, int depth, IGLMesh *mesh);

    void IntersectRay(Vector3d O, Vector3d D, IGLMesh *mesh, double *t, int *k, double *u, double *v);
    void GetFacesWithinRadius(Vector3d p, double r2, QSet<int> *fs, IGLMesh *mesh);

private:
    bool IntersectTriangleWithRay(Vector3d O, Vector3d D, int face_index, IGLMesh *mesh, double *t, double *u, double *v);
    bool BoxIntersectsRay(Vector3d O, Vector3d D, double t0, double t1);
    double SqDistToPoint(Vector3d p);
};

class AABBTree
{
public:
    AABBTree(void);
    AABBTree(IGLMesh *mesh);
    ~AABBTree(void);

    void IntersectRay(Vector3d O, Vector3d D, double *t, int *k, double *u, double *v);
    void GetFacesWithinRadius(Vector3d p, double r2, QSet<int> *fs);
    void rebuildTree();

    IGLMesh *theMesh;
    AABBNode *root;
};

