//
//  ParchmentFlattener.h
//  ParchmentFlattener
//
//  Copyright (c) 2014 Kazim Pal.
//  Released under the MIT License.
//
#ifndef ParchmentFlattener_H
#define ParchmentFlattener_H

#include <QtGui/QMainWindow>
#include "ui_parchmentflattener.h"
#include "FlatGLWidget.h"

class ParchmentFlattener : public QMainWindow
{
	Q_OBJECT

public:
    enum MarkupMode
    {
        None,
        Line,
        EditLine,
        Scale,
        EditScale,
        CrossField
    };

	ParchmentFlattener(QWidget *parent = 0, Qt::WFlags flags = 0);
	~ParchmentFlattener();

    static QString meshFilename;
    static QString meshDirectory;

    QTimer *wTimer;

    static MarkupMode markupMode;
    static bool verticalLines;
    void setMarkupMode(MarkupMode m);



    static bool inLineMode() { return ParchmentFlattener::markupMode == ParchmentFlattener::Line; }
    static bool inScaleMode() { return ParchmentFlattener::markupMode == ParchmentFlattener::Scale; }
    static bool inNoneMode() { return ParchmentFlattener::markupMode == ParchmentFlattener::None; }

public slots:
    void flatten();
    void setLineMode();
    void setScaleMode();
    void setNoneMode();
    void toggleVertical();
    void setVertical(bool isVertical);
    void saveConstraintFile();
    void loadConstraintFile();
    void clearConstraints();
    void loadMesh();
    void saveFlatMeshFile();

    void autosaveConstraintFile();

    void toggleLineMode();
    void toggleScaleMode();
    void nextConstraint();
    void prevConstraint();
    void deleteConstraint();
    void toggleRenderCrossField();

    void setRenderModeStandard();
    void setRenderModeStretch();

    void snapshot();

protected:
    virtual void keyPressEvent(QKeyEvent *e);

private:
	Ui::ParchmentFlattenerClass ui;
    //WrinkledGLWidget *wGlWidget;
	FlatGLWidget *fGlWidget;
    IGLMesh *mesh;
    QActionGroup *markupModeGroup;

    void initAll();
};

#endif // ParchmentFlattener_H
